package com.logicbig.example;

import com.querydsl.core.types.dsl.NumberExpression;
import org.springframework.data.querydsl.QuerydslPredicateExecutor;
import org.springframework.data.querydsl.binding.QuerydslBinderCustomizer;
import org.springframework.data.querydsl.binding.QuerydslBindings;
import org.springframework.data.repository.PagingAndSortingRepository;
import java.sql.Date;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public interface EmployeeRepository extends PagingAndSortingRepository<Employee, Long>,
        QuerydslPredicateExecutor<Employee>, QuerydslBinderCustomizer<QEmployee> {
    @Override
    default void customize(QuerydslBindings bindings, QEmployee root) {

        bindings.bind(root.salary)
                .first(NumberExpression::goe);

        bindings.bind(root.dateOfBirth)
                .all((path, value) -> {
                    List<? extends Date> dates = new ArrayList<>(value);
                    if (dates.size() == 1) {
                        return Optional.of(path.eq(dates.get(0)));
                    } else {
                        Date from = dates.get(0);
                        Date to = dates.get(1);
                        return Optional.of(path.between(from, to));
                    }
                });
    }
}